@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 
-# Copyright (c) 2010, Taher Haveliwala
+# Copyright (c) 2010, Taher Haveliwala <oss@taherh.org>
 # All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
@@ -37,8 +37,9 @@ sim(http://www.stanford.edu/,http://www.mit.edu/)=0.142787018368
 sim(http://www.berkeley.edu/,http://www.mit.edu/)=0.248877629741
 
 '''
-from __future__ import division, absolute_import, print_function
+from __future__ import division, absolute_import, print_function, unicode_literals
 
+import codecs
 import sys
 import math
 import re
@@ -47,13 +48,12 @@ import httplib2
 import lxml.html
 from lxml.html.clean import clean_html
 
-default_sim_func = None
-
 # --- top-level functions ---
 def measure_similarity(file_a, file_b, sim_func = None):
     '''
     Returns the textual similarity of file_a and file_b using chosen similarity metric
-    sim_func defaults to cosine_sim if not specified
+    'sim_func' defaults to cosine_sim if not specified
+    Consumes file_a and file_b
     '''
     if sim_func == None: sim_func = cosine_sim  # default to cosine_sim
     
@@ -62,15 +62,15 @@ def measure_similarity(file_a, file_b, sim_func = None):
     
     return sim_func(u, v)
 
-def pairwise_compare(files, sim_func = None):
+def pairwise_compare(filenames, sim_func = None):
     '''
-    Does a pairwise comparison of the entries in 'files' and prints similarities
+    Does a pairwise comparison of the entries in 'filenames' and prints similarities
     '''
-    print('Comparing files {0}'.format(str(files)))
-    for i in range(0,len(files)):
-        for j in range(i+1, len(files)):
-            fname_a = files[i]
-            fname_b = files[j]
+    print('Comparing files {0}'.format(str(filenames)))
+    for i in range(0,len(filenames)):
+        for j in range(i+1, len(filenames)):
+            fname_a = filenames[i]
+            fname_b = filenames[j]
             with get_text_stream(fname_a) as file_a:
                 with get_text_stream(fname_b) as file_b:
                     print('sim({0},{1})={2}'.
@@ -91,7 +91,7 @@ def jaccard_sim(A, B):
     Returns the Jaccard similarity of A,B: |A \cap B| / |A \cup B|
     We treat A and B as multi-sets (The Jaccard coefficient is technically defined over sets)
     '''
-    return mag_union(A, B) / mag_intersect(A, B)
+    return mag_intersect(A, B) / mag_union(A, B)
 
 # --- Term-vector operations ---
 
@@ -139,14 +139,12 @@ def term_vec(file):
     tf_dict = {}
     for line in file:
         for term in line.split():
-            if type(term) == bytes:
-                term = term.decode('utf-8')  # Make sure 'term' is in unicode (Python 2.x hack)
             if not term in tf_dict: tf_dict[term] = 0
             tf_dict[term] += 1
     return tf_dict
 
 # get_text_stream() needs an http object
-_h = httplib2.Http('.cache')
+_h = httplib2.Http(str('.cache'))  # httplib2 doesn't like unicode for cachefile name
 
 def get_text_stream(name):
     '''Returns a text stream from either a file or a url'''
@@ -160,9 +158,57 @@ def get_text_stream(name):
                                                encoding=unicode, method='text')
         file = io.StringIO(clean_html_string)
     else:
-        file  = open(name)
+        file  = codecs.open(name, encoding='utf-8')
     return file
 
+# --- Support for TFIDF weighting ---
+def parse_df(file):
+    '''
+    Parses a document frequency file for use in applying df term weighting
+    '''
+    df_dict = {}
+    for line in file:
+        ln_list = line.split()
+        if len(ln_list) == 0: continue  # skip blank lines without warning
+        if len(ln_list) != 2:  # raise exception if there were not exactly two entries in the line
+            raise FileFormatException(
+                'Bad line in doc freq file ({0} entries, expecting 2): {1}'.
+                format(len(ln_list), line))
+        (term, df) = ln_list
+        df_dict[term] = int(df)
+    return df_dict
+
+def write_df(df_dict, file):
+    '''
+    Writes the document frequency data structure to file
+    TODO: sort order?
+    '''
+    for (term, df) in df_dict.items():
+        file.write(u'{0}\t{1}\n'.format(term, df))
+    
+def compute_df(files):
+    '''
+    Creates a document frequency count by processing a collection of files
+    '''
+    df_dict = {}
+    for file in files:
+        term_seen = set()
+        for line in file:
+            for term in line.split():
+                if term not in term_seen:
+                    if term not in df_dict: df_dict[term] = 0
+                    df_dict[term]+=1
+                    term_seen.add(term)
+    return df_dict
+
+# --- Exceptions ---
+class Error(Exception):
+    '''Base class for Exception types used in this module'''
+    pass
+
+class FileFormatException(Exception):
+    pass
+    
 # --- main() ---
 
 def main():
